/*
   This file contains the main() function.

   You should not modify anything in this file (except
   for one include directive)..

   The main() function launches the thread that maintains
   the "console gui" and then main() goes into an infinite
   loop that creates one customer thread for every input
   character.
*/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "pthread.h"

typedef struct
{
   char id[4]; // each customer is given an id number
   int state;  // entry, dining, or exit states
   int time;   // how long this customer will be in the dining state
} customer_t;

#define CUSTOMERS 100  // maximum number of customer threads
customer_t* customers[CUSTOMERS];
int customerID = 10;  // initial customer id number

char inputChar;
pthread_t tid;

#include "printError.h"   // used to create readable error messages
#include "roomDisplay.h"  // this code maintains the "console gui"


// Four different implementations of the customer thread.
// You need to implement three of these,
// but use only one of them at a time.
#include "customer.h"              // provided by me
//#include "customerBounded.h"     // bound the number of dining customers
//#include "customerSushi.h"       // lock dining area when full
//#include "customerArtGallery.h"  // customers enter dining area as a group

int main(int argc, char *argv[])
{
   HANDLE hIn;
   DWORD mode;
   int numberRead, i;
   customer_t* newCustomer;

   // initialize the array of customers to NULL pointers
   for (i = 0; i < CUSTOMERS; i++)
      customers[i] = NULL;

   // Configure the console for raw character input.
   hIn = GetStdHandle(STD_INPUT_HANDLE);
   if (!GetConsoleMode(hIn, &mode))
      printError("GetConsoleMode");
   // disable input buffering and echoing
   mode = mode & ~(ENABLE_LINE_INPUT | ENABLE_ECHO_INPUT);
   if (!SetConsoleMode(hIn, mode))
      printError("SetConsoleMode");

   // Create the thread that updates the console window.
   pthread_create(&tid, NULL, roomDisplay, NULL);

   srand(time(NULL));  // seed the random number generator with current time

   while(1) // create one customer thread for each input character
   {
      // read a single input character (in raw mode) from the console
      if(!ReadFile(hIn, &inputChar, 1, (LPDWORD)&numberRead, NULL))
         printError("ReadFile");

      // create a customer_t struct for this thread
      newCustomer = (customer_t*)malloc(sizeof(customer_t));
      // give this customer a number
      itoa(customerID++, newCustomer->id, 10);
      // give this customer an amount of time for dining
      newCustomer->time = 5+(rand()%20);  // between 5 and 25 seconds
      // put this customer in the entry state
      newCustomer->state = 0;

      // Create a customer thread that will dine for a random time.
      pthread_create(&tid, NULL, customer, (void*)newCustomer);
   }

   return 0;
}//main()
