/*
   This file maintains the "console gui".

   You should not modify anything in this file (unless
   you want to make the "gui" more interesting).

   There is an array customers[] (declared in CS52500Hw1.c) that
   holds pointers to customer_t data structures, one structure
   for each running customer thread. The structure holds an id
   number, a state, and a run time for each thread. This gui
   walks through that array, and for each thread it finds, it
   draws a representation of that thread in the appropriate
   "room" of the gui (the state of the thread determines the
   appropriate room).
*/

// set the interval of time (in tenths of a second) between console updates
#define UPDATE_INTERVAL 1

// 80 columns by 25 rows is the default (smallest?) console window size?
#define COLUMNS 80
#define LINES 60

// This thread switches between the off-screen and on-screen buffers.
void* roomDisplay(void* threadParam)
{
   HANDLE hactive, hinactive, htemp; // handles for console image buffers
   COORD c1, c2, size;               // used for console image buffers
   SMALL_RECT rect;                  // used for console image buffers
   CONSOLE_SCREEN_BUFFER_INFO info;

   // character buffer used to erase the console image buffers
   char blankBuf[COLUMNS*LINES];
   int numberWritten;

   char entryRoom[]  = "Entry Room (strike any key to create a new cutomer) ========"; // 60 chars
   char diningRoom[] = "Dining Room ================================================";
   char exitRoom[]   = "Exit Room ==================================================";
   char customerString[9];

   int stateCounters[3];  // count the number of customers in each state

   int i, j;

   // get handle to console image buffer
   hactive = GetStdHandle(STD_OUTPUT_HANDLE);
   // create a second console image buffer for double buffering
   hinactive = CreateConsoleScreenBuffer(GENERIC_READ|GENERIC_WRITE, 0, 0, CONSOLE_TEXTMODE_BUFFER, 0);
   if (hinactive == INVALID_HANDLE_VALUE)
      printError("CreateConsoleScreenBuffer");
   // set the size of the two buffers
   size.X = COLUMNS;
   size.Y = LINES;
   if (!SetConsoleScreenBufferSize(hactive, size))
      printError("SetConsoleScreenBufferSize (hinactive)");
   if (!SetConsoleScreenBufferSize(hinactive, size))
      printError("SetConsoleScreenBufferSize (hactive)");

   // get the maximum size for a console window
   // (This method is inaccurate!!! See the documentation.)
   size = GetLargestConsoleWindowSize(hactive);
   if ( (size.X == 0) && (size.Y == 0) )
      printError("GetLargestConsoleWindowSize");

   // get the maximum size for a console window
   if (!GetConsoleScreenBufferInfo(hactive, &info))
      printError("GetConsoleScreenBufferInfo");
   size = info.dwMaximumWindowSize;

   // set the size of each console buffer's window
   // (make the console windows large enough to show the
   // whole buffer, or as large as possible, whichever is
   // smaller)
   rect.Left = 0;
   rect.Top = 0;
   rect.Right = (COLUMNS <= size.X) ? COLUMNS-1 : size.X-1;
   rect.Bottom = (LINES <= size.Y) ? LINES-1 : size.Y-1;
   if (!SetConsoleWindowInfo(hactive, TRUE, &rect))
       printError("SetConsoleWindowInfo (hactive)");
   if (!SetConsoleWindowInfo(hinactive, TRUE, &rect))
       printError("SetConsoleWindowInfo (hinactive)");

   // put lines of spaces in the blankBuf array
   for (i = 0; i < LINES; i++)
   {  for (j = 0; j < COLUMNS-1; j++)
         blankBuf[i*COLUMNS + j] = ' ';
      blankBuf[i*COLUMNS + j] = '\n';
   }
   blankBuf[COLUMNS*LINES-1] = '\0';  // null terminate the buffer

   // set the COORD strut to be the upper left hand corner (for the cursor)
   c1.X = 0;
   c1.Y = 0;

   // write blankBuf to the active console screen buffer
   if (!WriteConsole(hactive, blankBuf, LINES*COLUMNS, (LPDWORD)&numberWritten, NULL))
      printError("WriteConsole");
   if (!SetConsoleCursorPosition(hactive, c1))
      printError("SetConsoleCursorPosition-1");

   while(1)  // repeatedly swap the two console screen buffers
   {
      //
      c2.X = 0;
      c2.Y = 0;
      if (!WriteConsoleOutputCharacter(hinactive, entryRoom, sizeof(entryRoom),
                                        c2, (LPDWORD)&numberWritten))
         printError("WriteConsole");

      c2.X = 0;
      c2.Y = 20;
      if (!WriteConsoleOutputCharacter(hinactive, diningRoom, sizeof(entryRoom),
                                        c2, (LPDWORD)&numberWritten))
         printError("WriteConsole");

      c2.X = 0;
      c2.Y = 40;
      if (!WriteConsoleOutputCharacter(hinactive, exitRoom, sizeof(entryRoom),
                                        c2, (LPDWORD)&numberWritten))
         printError("WriteConsole");

      // walk the customer array, looking for customers to print out.
      stateCounters[0] = 0;
      stateCounters[1] = 0;  // count the number of customers in each state
      stateCounters[2] = 0;
      for (i = 0; i < CUSTOMERS; i++)
         if (customers[i])
         {
            stateCounters[ customers[i]->state ] += 1;
            c2.X = 8 * stateCounters[ customers[i]->state ];
            c2.Y = 20 * (customers[i]->state) + 5;
            // create an output string containing id number and time remaining in dining state
            sprintf(customerString, "%s(%d)", customers[i]->id, customers[i]->time);
            if (!WriteConsoleOutputCharacter(hinactive, customerString, 9,
                                              c2, (LPDWORD)&numberWritten))
               printError("WriteConsole");
         }

      c2.X = 79;
      c2.Y = 59;
      if (!WriteConsoleOutputCharacter(hinactive, &inputChar, 1, c2, (LPDWORD)&numberWritten))
         printError("WriteConsole");

      if (!SetConsoleCursorPosition(hinactive, c1))
         printError("SetConsoleCursorPosition-2");

      // move the scrolling information from one screen buffer to the other
      if (!GetConsoleScreenBufferInfo(hactive, &info))
         printError("GetConsoleScreenBufferInfo");
      if (!SetConsoleWindowInfo(hinactive, TRUE, &(info.srWindow)))
         printError("SetConsoleWindowInfo");
      // swap the two buffers
      htemp = hactive;
      hactive = hinactive;
      hinactive = htemp;
      // display the new active buffer
      if (!SetConsoleActiveScreenBuffer(hactive))
         printError("SetConsoleActiveScreenBuffer");

      // write blankBuf to the inactive console screen buffer
      if (!WriteConsole(hinactive, blankBuf, LINES*COLUMNS, (LPDWORD)&numberWritten, NULL))
         printError("WriteConsole (blanking)");
      if (!SetConsoleCursorPosition(hinactive, c1))
         printError("SetConsoleCursorPosition-3");

      Sleep(UPDATE_INTERVAL*100);
   }
   return 0;
}//roomDisplay()
